@@ -3,6 +3,7 @@ import fabio
 import datetime,os,copy
 from py4xs.mask import Mask
 from py4xs.local import ExpPara
+from py4xs.utils import calc_avg
 import pylab as plt
 import matplotlib as mpl
 from matplotlib.colors import LogNorm
@@ -16,22 +17,178 @@ class DataType(Enum):
     q = 4
     xyq = 5      # Cartesian version of qphi
     
+
+def get_bin_edges(grid):
+    """ find the bin edges that correspond to the desired data grid
+        the average of neghboring bin edges reproduces the position in the grid 
+    """
+    v0 = grid[0]-(grid[1]-grid[0])/2
+    edges = [v0]
+    for i in range(len(grid)):
+        v1 = grid[i]*2 - v0
+        edges.append(v1)
+        v0 = v1
+    return np.asarray(edges)
+
+def round_by_stepsize(v, ss):
+    prec = -int(np.floor(np.log10(np.fabs(ss))))
+    if prec<1:
+        prec = 1
+    return f"{v:.{prec}f}".rstrip('0')
+
+def grid_labels(grid, N=3, step_tol=0.2):
+    dd = np.diff(grid)
+    tt = []
+    td = []
+    glist = []
+    for i in range(len(dd)):
+        if len(tt)>0: 
+            if dd[i]-tt[-1]>step_tol*dd[i]:
+                glist.append([np.mean(tt), td])
+                tt = []  
+                td = []
+        tt.append(dd[i])
+        td.append(grid[i])
+    td.append(grid[i+1])
+    glist.append([np.mean(tt), td])
+    
+    if len(glist)==1: # single step size
+        ss = glist[0][0]
+        gpindex = np.linspace(1, len(grid), N, dtype=np.int)-1
+        gpvalues = [grid[k] for k in gpindex]
+        gplabels = [round_by_stepsize(gp,ss) for gp in gpvalues]
+    else:
+        gpindex = []
+        gpvalues = []
+        gplabels = []
+        i = 0
+        glist
+        for [ss, gp] in glist: 
+            gpindex.append(i)
+            gpvalues.append(gp[0])
+            gplabels.append(round_by_stepsize(gp[0],ss))
+            i += len(gp)
+        gpindex.append(i)
+        gpvalues.append(gp[-1])
+        gplabels.append(round_by_stepsize(gp[-1],ss))
+    
+    return gpindex,gpvalues,gplabels
+
 class MatrixWithCoords:
     # 2D data with coordinates
     d = None
     xc = None
+    xc_label = None
     yc = None 
+    yc_label = None
+    err = None
     datatype = None
     
     def copy(self):
         ret = MatrixWithCoords()
         ret.xc = self.xc
         ret.yc = self.yc
+        ret.xc_label = self.xc_label
+        ret.yc_label = self.yc_label
         ret.d = np.copy(self.d)
+        ret.err = self.err
         
         return ret
     
-    def conv(self, Nx1, Ny1, xc1, yc1, mask=None, cor_factor=1, datatype=DataType.det):
+    def expand(self, coord, axis, in_place=True):
+        """ extend x or y coordinates, as specified by axis and the new coordinates, so that dissimilar
+            datasets can be merged to produce a larger dataset
+        """
+        if axis=='x':
+            shape = (len(self.yc),len(coord))
+            d = np.full(shape, np.nan)
+            if self.err is not None:
+                err = np.full(shape, np.nan)
+            else:
+                err = None
+            for i in range(len(self.xc)):
+                i0 = np.where(coord==self.xc[i])[0][0]
+                d[:,i0] = self.d[:,i]
+                if err is not None:
+                    err[:,i0] = self.err[:,i]
+            if in_place:
+                self.xc = coord
+        elif axis=='y':
+            shape = (len(coord),len(self.xc))
+            d = np.full(shape, np.nan)
+            if self.err is not None:
+                err = np.full(shape, np.nan)
+            else:
+                err = None
+            for i in range(len(self.yc)):
+                i0 = np.where(coord==self.yc[i])[0][0]
+                d[i0,:] = self.d[i,:]
+                if err is not None:
+                    err[i0,] = self.err[i,:]
+            if in_place:
+                self.yc = coord
+        else:
+            raise Exception(f"invalid axis: {axis}")
+        
+        if not in_place:
+            return d,err
+
+        self.d = d
+        self.err = err
+        
+    def merge(self, ds):
+        """ merge a list of MatrixWithCoord together
+            all objects must have the same coordinates at least in one dimension
+
+            since it is difficult to check whether the coordinates are the identical (prec),
+            assume they are the same as long as the length are the same
+        """
+        common_x = True
+        common_y = True
+        for d in ds:
+            if self.xc_label!=d.xc_label or self.yc_label!=d.yc_label:
+                raise Exception("data to be merged have different x_labels.")
+
+        ret = MatrixWithCoords()
+        ret.xc = np.unique(np.hstack([m.xc for m in [self]+ds]).flatten())
+        ret.yc = np.unique(np.hstack([m.yc for m in [self]+ds]).flatten())
+        ret.xc_label = self.xc_label
+        ret.yc_label = self.yc_label
+        shape = (len(ret.yc),len(ret.xc))
+        
+        #print(common_x,common_y,len(ret.xc),len(ret.yc),shape)
+        
+        wt = np.zeros(shape)
+        ret.d = np.zeros(shape)
+        if self.err is not None:
+            ret.err = np.zeros(shape)
+        else:
+            ret.err = None
+        
+        idx = None
+        for d in [self]+ds:
+            # expand data if necessary
+            t = copy.copy(d)
+            if len(ret.xc)!=len(d.xc):
+                t.expand(ret.xc, "x")
+            if len(ret.yc)!=len(d.yc):
+                t.expand(ret.yc, "y")
+            idx = ~np.isnan(t.d)
+            ret.d[idx] += t.d[idx]
+            if ret.err is not None:
+                ret.err[idx] += t.err[idx]
+            wt[idx] += 1
+
+        idx = (wt>0)
+        ret.d[idx] /= wt[idx]
+        ret.d[~idx] = np.nan
+        if ret.err is not None:
+            ret.err[idx] /= wt[idx]
+            ret.err[~idx] = np.nan
+
+        return ret
+    
+    def conv0(self, Nx1, Ny1, xc1, yc1, mask=None, cor_factor=1, datatype=DataType.det):
         """ re-organize the 2D data based on new coordinates (xc1,yc1) for each pixel
             returns a new MatrixWithCoords, with the new coordinates specified by Nx1, Ny1
             Nx1, Ny1 can be either the number of bins or an array that specifies the bin edges
@@ -39,7 +196,7 @@ class MatrixWithCoords:
         """
         ret = MatrixWithCoords()
         ret.datatype = datatype
-        
+
         # correction factor is applied by dividing its value
         # should be called like this conv(...., cor_factor=exp.FSA)
         data = self.d/cor_factor
@@ -51,7 +208,12 @@ class MatrixWithCoords:
             xc1 = xc1[~(mask.map)].flatten()
             yc1 = yc1[~(mask.map)].flatten()
             data = data[~(mask.map)].flatten()
-        
+
+        if hasattr(Nx1, '__iter__'): # tuple or list or np array
+            Nx1 = get_bin_edges(Nx1)
+        if hasattr(Ny1, '__iter__'): # tuple or list or np array
+            Ny1 = get_bin_edges(Ny1)
+
         (v_map, x_edges, y_edges) = np.histogram2d(xc1, yc1,
                                                 bins=(Nx1, Ny1), weights=data)
         (c_map, x_edges, y_edges) = np.histogram2d(xc1, yc1,
@@ -63,9 +225,126 @@ class MatrixWithCoords:
         ret.d = np.fliplr(v_map/c_map).T
         ret.xc = (x_edges[:-1] + x_edges[1:])/2
         ret.yc = (y_edges[:-1] + y_edges[1:])/2
-        
         return ret
 
+    def conv(self, Nx1, Ny1, xc1, yc1, mask=None, interpolate=None, cor_factor=1, 
+             inc_stat_err=False, datatype=DataType.det, err_thresh=1e-5):
+        """ re-organize the 2D data based on new coordinates (xc1,yc1) for each pixel
+            returns a new MatrixWithCoords, with the new coordinates specified by Nx1, Ny1
+            Nx1, Ny1 can be either the number of bins or an array that specifies the bin edges
+            datatype is used to describe the type of the 2D data (detector image, qr-qz map, q-phi map)
+            
+            err_thresh is used to weed out some outliers
+        """
+        ret = MatrixWithCoords()
+        ret.datatype = datatype
+
+        # correction factor is applied by dividing its value
+        # should be called like this conv(...., cor_factor=exp.FSA)
+        data = self.d/cor_factor
+        if mask is None:
+            xc1 = xc1.flatten()
+            yc1 = yc1.flatten()
+            data = data.flatten()
+        else:
+            xc1 = xc1[~(mask.map)].flatten()
+            yc1 = yc1[~(mask.map)].flatten()
+            data = data[~(mask.map)].flatten()            
+
+        if hasattr(Nx1, '__iter__'): # tuple or list or np array
+            Nx1 = get_bin_edges(Nx1)
+        if hasattr(Ny1, '__iter__'): # tuple or list or np array
+            Ny1 = get_bin_edges(Ny1)
+
+        (c_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=np.ones(len(data)))
+        (xc_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=xc1)
+        (yc_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=yc1)
+        cidx = (c_map>0)
+
+        if inc_stat_err:
+            dw = np.zeros_like(data)
+            idx = (data!=0)    # to avoid the divergent weight for zero intensity
+            dw[idx] = 1./data[idx]
+            counted = np.zeros(len(data))
+            counted[idx] = 1.
+            (v_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=data*dw)
+            (w_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=dw)
+            # we are skipping the zero intensity pixels
+            (c1_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=counted)   
+            idx = (w_map>0)
+            v_map[idx] /= w_map[idx]
+            v_map[~idx] = np.nan
+            e_map = np.zeros_like(v_map)
+            e_map[idx] = 1./np.sqrt(w_map[idx])
+            e_map[~idx] = np.nan
+            # scale data based on how many pixels are counted vs should have been counted
+            scl = np.sqrt(c_map[idx]/c1_map[idx])
+            v_map[idx] /= scl
+            e_map[idx] /= scl
+        else:
+            (v_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=data)
+            (v2_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=data*data)
+            v_map[cidx] /= c_map[cidx]
+            v_map[~cidx] = np.nan
+            v2_map[cidx] /= c_map[cidx]
+            e_map = np.sqrt(v2_map-v_map*v_map)
+
+        ret.xc = (x_edges[:-1] + x_edges[1:])/2
+        ret.yc = (y_edges[:-1] + y_edges[1:])/2
+
+        xc_map[cidx] /= c_map[cidx]
+        yc_map[cidx] /= c_map[cidx]
+
+        ret.err = np.fliplr(e_map).T
+
+        # re-evaluate based on the actual coordinates instead of the expected
+        # unable to do it in 2D
+        # default is "x" for q in q-phi conversion
+        if interpolate=='x': 
+            for i in range(len(ret.yc)):
+                idx = np.isnan(v_map[:,i])
+                if (len(v_map[:,i][~idx])>2):
+                    v_map[:,i][~idx] = np.interp(ret.xc[~idx], xc_map[:,i][~idx], v_map[:,i][~idx])
+        elif interpolate=='y':
+            for i in range(len(ret.xc)):
+                idx = np.isnan(v_map[i,:])
+                if (len(v_map[i,:][~idx])>2):
+                    v_map[i,:][~idx] = np.interp(ret.yc[~idx], yc_map[i,:][~idx], v_map[i,:][~idx])
+
+        ret.d = np.fliplr(v_map).T
+        idx = (ret.d==0) | (ret.err<err_thresh)
+        ret.d[idx] = np.nan
+        ret.err[idx] = np.nan
+        return ret
+
+    def plot(self, ax=None, logScale=False, **kwargs):
+        if ax is None:
+            plt.figure()
+            ax = plt.gca()
+
+        if logScale:
+            ax.imshow(np.log(self.d), **kwargs)
+        else:
+            ax.imshow(self.d, **kwargs)
+        ax.set_xlabel('ix')
+        ax.set_ylabel('iy')
+
+        axx = ax.twiny()
+        gpindex,gpvalues,gplabels = grid_labels(self.xc)
+        axx.set_xticks(gpindex)
+        axx.set_xticklabels(gplabels)
+        if self.xc_label:
+            axx.set_xlabel(self.xc_label)
+
+        axy = ax.twinx()
+        gpindex,gpvalues,gplabels = grid_labels(self.yc)
+        axy.set_yticks(gpindex)
+        axy.set_yticklabels(gplabels)
+        if self.yc_label:
+            axy.set_ylabel(self.yc_label)
+            
+        axy.format_coord = ax.format_coord #make_format(ax2, ax1)
+
     def roi(self, x1, x2, y1, y2, mask=None):
         """ return a ROI within coordinates of x=x1~x2 and y=y1~y2 
         """
@@ -79,7 +358,11 @@ class MatrixWithCoords:
 
         ret.xc = self.xc[xidx]
         ret.yc = self.yc[yidx]
+        ret.xc_label = self.xc_label
+        ret.yc_label = self.yc_label
         ret.d = np.asarray(self.d[t1*t2].reshape((len(ret.yc),len(ret.xc))), dtype=float)
+        if self.err is not None:
+            ret.err = np.asarray(self.err[t1*t2].reshape((len(ret.yc),len(ret.xc))), dtype=float)
         
         if mask is not None:
             idx = mask.map[t1*t2].reshape((len(ret.yc),len(ret.xc)))
@@ -140,46 +423,52 @@ class MatrixWithCoords:
         
         return ret
     
-    def flatten(self, axis=0):
-        """ collapse the matrix into an array 
-            along x coordinates if axis=0, along y coordinates if axis=1
+    def flatten(self, axis='x', method='err_weighted'):
+        """ collapse the matrix onto the specified axis and turn it into an array 
         """
-        cor = np.ones(self.d.shape)
-        data = np.copy(self.d)
+        if not axis in ['x', 'y']:
+            raise Exception(f"unkown axis for flattening data: {axis}")
+        if not method in ['simple', 'err_weighted']:
+            raise Exception(f"unknow method for averaging: {method}")
+        if method=="err_weighted" and self.err is None:
+            raise Exception(f"err_weighted averaging requires error bar data.")
+    
+        dat = []
+        err = []
+        if axis=='x':
+            for i in range(len(self.yc)):
+                dat.append(self.d[i,:])
+                if self.err is not None:
+                    err.append(self.err[i,:])
+        if axis=='y':
+            for i in range(len(self.xc)):
+                dat.append(self.d[:,i])
+                if self.err is not None:
+                    err.append(self.err[:,i])
+        if self.err is None:
+            err = None
+        dd,ee = calc_avg(dat, err, method=method)
         
-        idx = np.isnan(data)
-        cor[idx] = 0
-        data[idx] = 0
-        
-        if axis!=0 and axis!=1:
-            raise Exception("unkown axis for flattening data.")
-        
-        # exclude blank portion of the data
-        dcor = np.sum(cor, axis=axis)
-        dd = np.sum(data, axis=axis)
-        idx = (dcor<=0)
-        dcor[idx] = 1.
-        dd[idx] =np.nan
-        dd /= dcor
+        return dd,ee
 
-        if axis==0:
-            return dd 
-        elif axis==1:
-            # the data is always stored with the upper left corner first in the memory
-            # the index is stored with 
-            return np.flipud(dd)
-
-    def average(self, datalist):
+    def average(self, datalist:list, weighted=False):
         """ average with the list of data given
             all data must have the same coordinates and datatype
         """
-        ret = copy.deepcopy(self)
         for d in datalist:
             if not np.array_equal(ret.xc, d.xc) or not np.array_equal(ret.xc, d.xc) or ret.datatype!=d.datatype:
-                raise Exception("attempted average between imcompatiple data.")
-            ret.d += d.d
-        ret.d /= (len(datalist)+1)
-        
+                raise Exception("attempted average between incompatiple data.")
+            if weighted and d.err is None:
+                raise Exception("weighted average requires error bars for each dataset.")
+
+        ret = copy.deepcopy(self)
+        dat = [d.d for d in [self]+datalist]
+        if weighted:
+            err = [d.err for d in [self]+datalist]
+            calc_avg(dat, err, "")
+        else: 
+            calc_avg(dat)
+            
         return ret
     
     def bkg_cor(self, dbkg, scale_factor=1.0, in_place=True):
@@ -187,14 +476,13 @@ class MatrixWithCoords:
             raise Exception("attempted background subtraction using imcompatiple data.")
         if in_place:
             self.d -= np.asarray(dbkg.d*scale_factor, dtype=self.d.dtype)
+            if self.err and dbkg.err:
+                self.err += dbkg.err*scale_factor
         else:
             ret = copy.deepcopy(self)
             ret.d = self.d - dbkg.d*scale_factor
-    
-    def merge(self, datalist):
-        """ merge with the list of data given, not necessarily having the same coordinates
-        """
-        pass 
+            if self.err and dbkg.err:
+                ret.err = self.err + dbkg.err*scale_factor
 
 
 class Data2d:
@@ -202,7 +490,7 @@ class Data2d:
         stores the scattering pattern itself, 
     """
 
-    def __init__(self, img, timestamp=None, uid='', exp=None, label='', dtype=None):
+    def __init__(self, img, timestamp=None, uid='', exp=None, label='', dtype=None, flat=None):
         """ read 2D scattering pattern
             img can be either a filename (rely on Fabio to deal with the file format) or a numpy array 
         """
@@ -360,7 +648,9 @@ class Data2d:
         qq[idx1] /= norm[idx1]
         Iq[idx1] /= norm[idx1]
         Iq2[idx1] /= norm[idx1]
-        dI = np.sqrt(Iq2-Iq*Iq)
+        dI = Iq2-Iq*Iq
+        dI[dI<0] = 0  # sometimes get runtime warnings
+        dI = np.sqrt(dI)
         dI[idx1] /= np.sqrt(norm[idx1])
         qq[~idx1] = np.nan
         Iq[~idx1] = np.nan
@@ -541,6 +831,8 @@ class Axes2dPlot:
         else: # unknown standard
             return 
         self.mark_coords(q_std, [], sym, DataType.q)
+        if self.exp and self.datatype==DataType.det:
+            self.mark_points([self.exp.bm_ctr_x], [self.exp.bm_ctr_y], datatype=DataType.det)
     
     # simply plot the x,y coordinates
     def mark_line(self, xv, yv, fmt="r-"):
