@@ -11,7 +11,7 @@ import multiprocessing as mp
 from py4xs.slnxs import Data1d,average,filter_by_similarity,trans_mode,estimate_scaling_factor
 from py4xs.utils import common_name,max_len,Schilling_p_value
 from py4xs.detector_config import create_det_from_attrs
-from py4xs.local import det_names,det_model,beamline_name    # e.g. "_SAXS": "pil1M_image"
+from py4xs.local import det_names,det_model,beamline_name,incident_monitor_field,transmitted_monitor_field
 from py4xs.data2d import Data2d,Axes2dPlot,MatrixWithCoords,DataType
 from py4xs.utils import run
 from itertools import combinations
@@ -57,11 +57,16 @@ def create_linked_files(fn, fnlist):
         fs.close()
     ff.close()
 
-def integrate_mon(em, ts, ts0, exp):
+def integrate_mon(em, ts, ts0, exp, extend_mon_stream):
     """ integrate monitor counts
         monitor counts are given by em with timestamps ts
         ts0 is the timestamps on the exposures, with duration of exp
+        
+        assume ts and ts0 are 1d arrays
     """
+    if extend_mon_stream and ts[-1]<ts0[-1]+exp*1.5:
+        em = np.append(em, em[-1])
+        ts = np.append(ts, ts0[-1]+exp*1.5)
     ffe = interp1d(ts, em)
     em0 = []
     for t in ts0:
@@ -69,7 +74,32 @@ def integrate_mon(em, ts, ts0, exp):
         ee = ffe(tt)
         em0.append(simpson(ee, tt))
     return np.asarray(em0)/exp
+
+def get_monitor_counts(grp, fieldName):
+    """ look under a data group (grp) that belong to a specific sample, find the stream that contains fieldName
+        caluclate the monitor counts based on the given timestamps (ts) and exposure time
+    """
+    strn = None
+    for stream in list(grp):
+        if not 'data' in list(grp[stream]):
+            continue
+        if fieldName in list(grp[stream]["data"]):
+            strn = stream
+            break
+    if strn is None:
+        raise Exeption(f"could not find the stream that contains {fieldName}.")
     
+    data = grp[strn]["data"][fieldName][...]
+    ts = grp[strn]["timestamps"][fieldName][...]
+
+    return strn,ts,data
+
+
+
+
+
+
+
 def pack_d1(data, ret_trans=True):
     """ utility function to creat a list of [intensity, error] from a Data1d object 
         or from a list of Data1s objects
@@ -101,12 +131,7 @@ def unpack_d1(data, qgrid, label, trans_value):
         ret.label = label
         ret.set_trans(trans_mode.external, trans_value)  # TODO: save transMode of d1s when packing 
         return ret
-
-def pack_d2(data):
-    pass
-
-def unpack_d2(data):
-    pass
+    
     
 def merge_d1s(d1s, detectors, save_merged=False, debug=False):
     """ utility function to merge 1D data sets, using functions under slnxs 
@@ -119,16 +144,23 @@ def merge_d1s(d1s, detectors, save_merged=False, debug=False):
     d_min = np.zeros(s0.qgrid.shape)+1.e32
     e_tot = np.zeros(s0.qgrid.shape)
     c_tot = np.zeros(s0.qgrid.shape)
+    w_tot = np.zeros(s0.qgrid.shape)
     label = None
     comments = ""
                 
     for d1 in d1s:        
         # empty part of the data is nan
         idx = ~np.isnan(d1.data)
-        d_tot[idx] += d1.data[idx]
-        e_tot[idx] += d1.err[idx]
+        # simple averaging
+        #d_tot[idx] += d1.data[idx]
+        #e_tot[idx] += d1.err[idx]
         c_tot[idx] += 1
-
+        # average using 1/sigma as weight
+        wt = 1/d1.err[idx]**2
+        d_tot[idx] += wt*d1.data[idx]
+        e_tot[idx] += d1.err[idx]**2*wt**2
+        w_tot[idx] += wt
+        
         idx1 = (np.ma.fix_invalid(d1.data, fill_value=-1)>d_max).data
         d_max[idx1] = d1.data[idx1]
         idx2 = (np.ma.fix_invalid(d1.data, fill_value=1e32)<d_min).data
@@ -139,15 +171,18 @@ def merge_d1s(d1s, detectors, save_merged=False, debug=False):
             label = d1.label
         else:
             label = common_name(label, d1.label)
-        
-    s0.data = d_tot
-    s0.err = e_tot
+    
+    # simple averaging
+    #s0.data[idx] /= c_tot[idx]
+    #s0.err[idx] /= np.sqrt(c_tot[idx])
+    # averaging by weight
+    s0.data = d_tot/w_tot
+    s0.err = np.sqrt(e_tot)/w_tot
     idx = (c_tot>1)
     s0.overlaps.append({'q_overlap': s0.qgrid[idx],
                         'raw_data1': d_max[idx],
                         'raw_data2': d_min[idx]})
-    s0.data[idx] /= c_tot[idx]
-    s0.err[idx] /= np.sqrt(c_tot[idx])
+    
     s0.label = label
     s0.comments = comments # .replace("# ", "## ")
     if save_merged:
@@ -439,9 +474,12 @@ class h5exp():
 
         self.save_detectors()
 
-def find_field(fh5, fieldName):
+def find_field(fh5, fieldName, sname=None):
     tstream = {}
-    samples = lsh5(fh5, top_only=True, silent=True)
+    if sname is None:
+        samples = lsh5(fh5, top_only=True, silent=True)
+    else:
+        samples = [sname]
     for sn in samples:
         for stream in list(fh5[f"{sn}"]):
             if not 'data' in list(fh5[f"{sn}/{stream}"]):
@@ -449,7 +487,10 @@ def find_field(fh5, fieldName):
             if fieldName in list(fh5[f"{sn}/{stream}/data"]):
                 tstream[sn] = stream
                 break
-    return tstream
+    if sname is None:
+        return tstream
+    
+    return tstream[sname] 
         
 class h5xs():
     """ Scattering data in transmission geometry
@@ -457,12 +498,15 @@ class h5xs():
         Data processing can be done either in series, or in parallel. Serial processing can be forced.
         
     """    
-    def __init__(self, fn, exp_setup=None, transField='', save_d1=True):
+    def __init__(self, fn, exp_setup=None, transField=transmitted_monitor_field, save_d1=True, 
+                 have_raw_data=True, read_only=False):
         """ exp_setup: [detectors, qgrid]
             transField: the intensity monitor field packed by suitcase from databroker
             save_d1: save newly processed 1d data back to the h5 file
         """
+        self.d0s = {}
         self.d1s = {}
+        self.d2s = {}
         self.detectors = None
         self.samples = []
         self.attrs = {}
@@ -471,64 +515,73 @@ class h5xs():
         self.transStream = {}  
 
         self.fn = fn
-        self.save_d1 = save_d1
-        self.fh5 = h5py.File(self.fn, "r+")   # file must exist
+        if read_only:
+            self.fh5 = h5py.File(self.fn, "r")   # file must exist
+        else:
+            self.fh5 = h5py.File(self.fn, "r+")   # file must exist
         if exp_setup==None:     # assume the h5 file will provide the detector config
             self.qgrid = self.read_detectors()
         else:
             self.detectors, self.qgrid = exp_setup
-            self.save_detectors()
+            if not read_only:
+                self.save_detectors()
         self.list_samples(quiet=True)
 
-        # find out what are the fields corresponding to the 2D detectors
-        # at LiX there are two possibilities; assume all samples have have data stored in the same fileds
-        sn = self.samples[0]
-        streams = list(self.fh5[f"{sn}"])
-        data_fields = {}
-        for stnm in streams:
-            if 'data' in self.fh5[f"{sn}/{stnm}"].keys(): 
-                for tf in list(self.fh5[f"{sn}/{stnm}/data"]):
-                    data_fields[tf] = stnm
+        if have_raw_data:
+            # find out what are the fields corresponding to the 2D detectors
+            # at LiX there are two possibilities; assume all samples have have data stored in the same fileds
+            self.save_d1 = save_d1
+            sn = self.samples[0]
+            streams = list(self.fh5[f"{sn}"])
+            data_fields = {}
+            for stnm in streams:
+                if 'data' in self.fh5[f"{sn}/{stnm}"].keys(): 
+                    for tf in list(self.fh5[f"{sn}/{stnm}/data"]):
+                        data_fields[tf] = stnm
         
-        self.det_name = None
-        # these are the detectors that are present in the data
-        d_dn = [d.extension for d in self.detectors]
-        for det_name in det_names:
-            for k in set(det_name.keys()).difference(d_dn):
-                del det_name[k]
-            if set(det_name.values()).issubset(data_fields.keys()):
-                self.det_name = det_name
-                break
-        if self.det_name is None:
-            print('fields in the h5 file: ', data_fields)
-            raise Exception("Could not find the data corresponding to the detectors.")
+            self.det_name = None
+            # these are the detectors that are present in the data
+            d_dn = [d.extension for d in self.detectors]
+            for det_name in det_names:
+                dn = copy.copy(det_name)
+                for k,v in det_name.items():
+                    if not k in d_dn or not v in data_fields.keys():
+                        del dn[k]
+                if len(dn.keys())>0:
+                    self.det_name = dn
+                    break
 
-        # transStream is more complicated
-        # different samples may store the data in different streams 
-        if transField=='': 
-            if 'trans' in self.fh5.attrs:
-                tf = self.fh5.attrs['trans'].split(',')
-                # transMove, transField, transStream 
-                # but transStream is not always recorded
-                v, self.transField = tf[:2]
-                self.transMode = trans_mode(int(v))
-                self.transStream = find_field(self.fh5, self.transField)
-                return
+            if self.det_name is None:
+                print('fields in the h5 file: ', data_fields)
+                raise Exception("Could not find the data corresponding to the detectors.")
+
+            # transStream is more complicated
+            # different samples may store the data in different streams 
+            if transField=='': 
+                if 'trans' in self.fh5.attrs:
+                    tf = self.fh5.attrs['trans'].split(',')
+                    # transMove, transField, transStream 
+                    # but transStream is not always recorded
+                    v, self.transField = tf[:2]
+                    self.transMode = trans_mode(int(v))
+                    self.transStream = find_field(self.fh5, self.transField)
+                    return
+                else:
+                    self.transMode = trans_mode.from_waxs
+                    self.transField = ''
+                    self.transStream = {}
             else:
-                self.transMode = trans_mode.from_waxs
-                self.transField = ''
-                self.transStream = {}
-        else:
-            try:
-                self.transStream = find_field(self.fh5, transField)
-            except:
-                print("invalid field for transmitted intensity: ", transField)
-                raise Exception()
-            self.transField = transField
-            self.transMode = trans_mode.external
-            
-        self.fh5.attrs['trans'] = ','.join([str(self.transMode.value), self.transField])  #elf.transStream])
-        self.fh5.flush()
+                try:
+                    self.transStream = find_field(self.fh5, transField)
+                except:
+                    print("invalid field for transmitted intensity: ", transField)
+                    raise Exception()
+                self.transField = transField
+                self.transMode = trans_mode.external
+
+            if not read_only:
+                self.fh5.attrs['trans'] = ','.join([str(self.transMode.value), self.transField])  #elf.transStream])
+                self.fh5.flush()
             
     def save_detectors(self):
         dets_attr = [det.pack_dict() for det in self.detectors]
@@ -542,6 +595,13 @@ class h5xs():
         self.detectors = [create_det_from_attrs(attrs) for attrs in json.loads(dets_attr)]  
         return np.asarray(qgrid)
 
+    def dset(self, dsname, data_type="data", sn=None):
+        assert (data_type in ["data", "timestamps"])
+        if sn is None:
+            sn = self.samples[0]
+        strm = find_field(self.fh5, dsname, sn)
+        return self.fh5[f"{sn}/{strm}/{data_type}/{dsname}"]
+    
     def md_dict(self, sn, md_keys=[]):
         """ create the meta data to be recorded in ascii data files
             from the detector_config (attribute of the h5xs object) 
@@ -681,7 +741,10 @@ class h5xs():
             sn = self.samples[0]
         d2s = {}
         for det in self.detectors:
-            dset = self.fh5["%s/primary/data/%s" % (sn, self.det_name[det.extension])]
+            try:
+                dset = self.fh5[f"{sn}/primary/data/{self.det_name[det.extension]}"]
+            except:
+                continue
             frn = self.verify_frn(sn, frn)
             d2 = Data2d(dset[tuple(frn)], exp=det.exp_para, dtype=dtype)
             d2.md["frame #"] = frn 
@@ -797,8 +860,9 @@ class h5xs():
         plt.show() 
     
     def show_data_qxy(self, sn=None, frn=None, ax=None, dq=0.006,
-                      fig_type="qxy", apply_sym=False, fix_gap=False,
-                      logScale=True, useMask=True, clim=(0.1,14000), showRef=True, cmap=None, dtype=None):
+                      fig_type="qxy", apply_sym=False, fix_gap=False, bkg=None,
+                      logScale=True, useMask=True, clim=(0.1,14000), showRef=True, 
+                      aspect='auto', cmap=None, dtype=None, colorbar=False):
         """ display frame #frn of the data under det for sample sn
             det is a list of detectors, or a string, data file extension
             fig_type should be either "qxy" or "qphi"
@@ -822,31 +886,46 @@ class h5xs():
         yqgrid = np.arange(start=yqmin, stop=yqmax+dq, step=dq)        
 
         xyqmaps = []
-        for dn in d2s.keys():   
+        for det in self.detectors:
+            dn = det.extension
             if useMask:
                 mask = d2s[dn].exp.mask
             else:
                 mask = None
-            mp = d2s[dn].data.conv(xqgrid, yqgrid, 
-                                   d2s[dn].exp.xQ, d2s[dn].exp.yQ, 
-                                   mask=mask)
-            mp.d *= (d2s[dn].exp.Dd/d2s["_SAXS"].exp.Dd)**2
-            xyqmaps.append(mp.d)
+            
+            cor_factor = d2s[dn].exp.FSA*d2s[dn].exp.FPol
+            if det.flat is not None:
+                cor_factor = det.flat*cor_factor
+            dm = d2s[dn].data.conv(xqgrid, yqgrid, d2s[dn].exp.xQ, d2s[dn].exp.yQ, 
+                                   mask=mask, cor_factor=cor_factor)
+            
+            if bkg is not None:
+                if dn in bkg.keys():
+                    dbkg = Data2d(bkg[dn], exp=det.exp_para)
+                    dm_b = dbkg.data.conv(xqgrid, yqgrid, d2s[dn].exp.xQ, d2s[dn].exp.yQ, 
+                                          mask=mask, cor_factor=cor_factor)
+                    dm.d -= dm_b.d    
+            
+            dm.d *= (d2s[dn].exp.Dd/d2s["_SAXS"].exp.Dd)**2
+            xyqmaps.append(dm.d)
+        
         xyqmap = merge(xyqmaps)
-
         if logScale:
-            plt.imshow(np.log(xyqmap), extent=(xqmin, xqmax, yqmin, yqmax))
-            plt.clim(np.log(clim))
+            im = ax.imshow(np.log(xyqmap), extent=(xqmin, xqmax, yqmin, yqmax), aspect=aspect, cmap=cmap)
+            im.set_clim(np.log(clim))
         else:
-            plt.imshow(xyqmap, extent=(xqmin, xqmax, yqmin, yqmax))
-            plt.clim(clim)
+            im = ax.imshow(xyqmap, extent=(xqmin, xqmax, yqmin, yqmax), aspect=aspect, cmap=cmap)
+            im.set_clim(clim)
 
         ax.set_title(f"frame #{d2s[list(d2s.keys())[0]].md['frame #']}")
+        if colorbar:
+            plt.colorbar(im)
 
 
     def show_data_qphi(self, sn=None, frn=None, ax=None, Nq=200, Nphi=60,
                        apply_symmetry=False, fill_gap=False, interp_method='linear',
-                       logScale=True, useMask=True, clim=(0.1,14000), showRef=True, cmap=None, dtype=None):
+                       logScale=True, useMask=True, clim=(0.1,14000), showRef=True, bkg=None,
+                       aspect="auto", cmap=None, dtype=None, colorbar=False):
         d2s = self.get_d2(sn=sn, frn=frn, dtype=dtype)
         if ax is None:
             plt.figure()
@@ -870,20 +949,37 @@ class h5xs():
         phi_grid = np.linspace(-180., 180, Nphi)
 
         dms = []
-        for dn in d2s.keys():   
+        for det in self.detectors:
+            dn = det.extension
+            if not dn in d2s.keys():
+                continue
             if useMask:
                 mask = d2s[dn].exp.mask
             else:
                 mask = None
+            
+            cor_factor = d2s[dn].exp.FSA*d2s[dn].exp.FPol
+            if det.flat is not None:
+                cor_factor = det.flat*cor_factor
+                
             # since the q/phi grids are specified, use the MatrixWithCoord.conv() function
             # the grid specifies edges of the bin for histogramming
             # the phi value in exp_para may not always be in the range of (-180, 180)
-            dm = d2s[dn].data.conv(q_grid, phi_grid, 
-                                   d2s[dn].exp.Q, 
+            dm = d2s[dn].data.conv(q_grid, phi_grid, d2s[dn].exp.Q, 
                                    fix_angular_range(d2s[dn].exp.Phi),
+                                   cor_factor=cor_factor,
                                    mask=mask, datatype=DataType.qphi)
+            
+            if bkg is not None:
+                if dn in bkg.keys():
+                    dbkg = Data2d(bkg[dn], exp=det.exp_para)
+                    dm_b = dbkg.data.conv(q_grid, phi_grid, d2s[dn].exp.Q,
+                                          fix_angular_range(d2s[dn].exp.Phi),
+                                          cor_factor=cor_factor,
+                                          mask=mask, datatype=DataType.qphi)
+                    dm.d -= dm_b.d    
 
-            d1 = dm.d*(d2s[dn].exp.Dd/d2s["_SAXS"].exp.Dd)**2
+            d1 = dm.d/det.fix_scale
             if apply_symmetry:
                 Np = int(Nphi/2)
                 d2 = np.vstack([d1[Np:,:], d1[:Np,:]])
@@ -896,17 +992,117 @@ class h5xs():
                 interp_d2(d1, method=interp_method)
 
             dms.append(d1)
-
+        
         qphimap = merge(dms)
         if logScale:
-            plt.imshow(np.log(qphimap), extent=(qmin, qmax, -180, 180), aspect="auto")
-            plt.clim(np.log(clim))
+            im = ax.imshow(np.log(qphimap), extent=(qmin, qmax, -180, 180), aspect=aspect, cmap=cmap)
+            im.set_clim(np.log(clim))
         else:
-            plt.imshow(qphiqmap, extent=(qmin, qmax, -180, 180), aspect="auto")
-            plt.clim(clim)
+            im = ax.imshow(qphiqmap, extent=(qmin, qmax, -180, 180), aspect=aspect, cmap=cmap)
+            im.set_clim(clim)
 
         ax.set_title(f"frame #{d2s[list(d2s.keys())[0]].md['frame #']}")
+        if colorbar:
+            plt.colorbar(im)
 
+    def get_mon(self, sn=None, trigger=None, gf_sigma=2, exp=1, 
+                force_synch=-0.25, force_synch_trig=0, extend_mon_stream=True,
+                debug=False, plot_trigger=False, **kwargs): 
+        """ calculate the monitor counts for each data point
+            1. if the monitors are read together with the detectors 
+            2. if the monitors are used asynchronously, monitor values would need to be integated 
+               based on timestampls; a trigger must be provided 
+                2a: in fly scans, this should be a motor name
+                2b: for solution scattering, use "sol" as trigger
+
+            the timestamps on the trigger and em1/em2 may not be synced, dt0 provides a correction 
+            to the trigger timestamp
+            
+            if plot_trigger=True and a single sample is named as sn, generate a plot for verification
+
+            timestamps on em1 appear to be way off, ntpd not running
+            if force_synch is non-zero, use first em2 timestamp + force_synch as start of em1 
+            
+            in some cases, the monitor stream appears to end before data collection is complete
+            if extend_mon_stream=True, repeat the last mon data point until the timestamp covers
+            the range of the trigger
+        """
+        if sn is None or isinstance(sn, list):
+            if plot_trigger:
+                plot_trigger = False           # plot for a single sample only
+                print("Disabling plot_trigger since not a single sample name is specified.")
+            samples = self.samples
+        else:
+            samples = [sn]
+           
+        for sn in samples:
+            if "pilatus" in self.header(sn).keys():
+                md = self.header(sn)['pilatus']
+                if 'exposure_time' in md.keys():
+                    exp=md['exposure_time']
+
+            # expect trans and incid monitor data to be in the same stream
+            strn,ts2,trans_data = get_monitor_counts(self.fh5[sn], transmitted_monitor_field)
+            strn,ts1,incid_data = get_monitor_counts(self.fh5[sn], incident_monitor_field)
+            if force_synch!=0: # timestamps between em1/em2 
+                ts1 = ts1-ts1[0]+ts2[0]+force_synch
+                
+            if strn=="primary":
+                print("monitors are used as detectors.")
+            elif trigger=="sol" or trigger is None:
+                dn = list(self.det_name.values())[0]
+                # expect a finite but minimal offset in time since all come from the same IOC server
+                ts0 = self.fh5[f'{sn}/primary/timestamps/{dn}'][...].flatten()
+                dshape = self.fh5[f"{sn}/primary/data/{dn}"].shape[0]   # length of the time sequence
+                if len(ts0)==1: # multiple exposures, single trigger, as in HT measurements
+                    ts0 = ts0[0]+np.arange(dshape)*exp    
+            elif trigger in self.fh5[f'{sn}/primary/timestamps'].keys():
+                ts0 = self.fh5[f'{sn}/primary/timestamps/{trigger}'][...].flatten()
+                dshape = self.fh5[f"{sn}/primary/data/{list(self.det_name.values())[0]}"].shape[:-2]
+                if len(dshape)>1:
+                    if len(dshape)>2:
+                        raise Exception(f"Don't know how to handle data shape {dshape}")
+                    dshape = dshape[0]*dshape[1]
+                else:
+                    dshape = dshape[0]
+                if len(ts0) != dshape:
+                    raise Exception(f"mistached timestamp length: {len(ts0)} vs {dshape}")
+                if len(ts0)>1: # expect the monitor data to be 1D
+                    ts0 = ts0.flatten()
+            else:
+                raise Exception(f"timestamp data for {trigger} cannot be found.")
+
+            if strn=="primary":
+                trans_data0 = trans_data
+                incid_data0 = incid_data
+            else:
+                try:
+                    trans_data0 = integrate_mon(trans_data, ts2, ts0+force_synch_trig, exp, extend_mon_stream)
+                    incid_data0 = integrate_mon(incid_data, ts1, ts0+force_synch_trig, exp, extend_mon_stream)                
+                except:
+                    t0 = np.min(ts2)
+                    print(f"time series likely misaligned:")
+                    print(f"trans mon: {np.min(ts2)-t0} ~ {np.max(ts2)-t0}")
+                    print(f"incid mon: {np.min(ts1)-t0} ~ {np.max(ts1)-t0}")
+                    print(f"detector: start={ts0-t0}, force_synch={force_synch_trig}, exp = {exp}")
+                    raise
+
+                if plot_trigger:
+                    plt.figure()
+                    plt.plot(ts2, trans_data/np.max(trans_data))
+                    plt.plot(ts0+force_synch_trig, trans_data0/np.max(trans_data), "o")
+                    plt.plot(ts1, incid_data/np.max(incid_data))
+                    plt.plot(ts0+force_synch_trig, incid_data0/np.max(incid_data), "o")
+                    
+            if not hasattr(self, "d0s"):
+                self.d0s = {}
+            if not sn in self.d0s.keys():
+                self.d0s[sn] = {}
+            self.d0s[sn]["transmitted"] = trans_data0
+            self.d0s[sn]["incident"] = incid_data0
+            transmission = trans_data0/incid_data0
+            transmission /= np.nanmean(transmission)
+            self.d0s[sn]["transmission"] = transmission
             
     def set_trans(self, transMode, sn=None, trigger=None, gf_sigma=2, dt0=-133.8, exp=1, plot_trigger=False, **kwargs): 
         """ set the transmission values for the merged data
@@ -939,43 +1135,12 @@ class h5xs():
         for s in samples:
             if self.transMode==trans_mode.external: # transField should be set/validated already
                 assert(self.transField!='')
-                trans_data = self.fh5[f'{s}/{self.transStream[s]}/data/{self.transField}'][...]
-                ts = self.fh5[f'{s}/{self.transStream[s]}/timestamps/{self.transField}'][...]
-                if self.transStream[s]!="primary": 
-                    # fly scanning, trigger must be a motor or "sol"  
-                    if trigger is None:
-                        raise Exception("the motor that triggers data collection must be specified.")
-                    elif trigger=="sol":
-                        # in this case the monitors are read more frequently than detector exposures
-                        # integrate monitor counts based  
-                        dn = list(self.det_name.values())[0]
-                        # expect a finite but minimal offset in time since all come from the same IOC server
-                        ts0 = self.fh5[f'{s}/primary/timestamps/{dn}'][...]-dt0    
-                        dshape = self.fh5[f"{s}/primary/data/{dn}"].shape[0]   # length of the time sequence
-                        md = self.header(s)['pilatus']
-                        if 'exposure_time' in md.keys():
-                            exp=md['exposure_time']
-                        if len(ts0)==1: # multiple exposures, single trigger, as in HT measurements
-                            ts0 = ts0[0]+np.arange(dshape)*exp    
-                        trans_data1 = integrate_mon(trans_data, ts, ts0, exp)
-                    elif trigger in self.fh5[f'{s}/primary/timestamps'].keys():
-                        # intepolate instead of integrate, with filtering, less accurate
-                        trigger_ts = sef.fh5[f'{s}/primary/timestamps/{trigger}'][...]-dt0
-                        dshape = self.fh5[f"{s}/primary/data/{list(self.det_name.values())[0]}"].shape[:-2]
-                        if trigger_ts.shape != dshape:
-                            raise Exception(f"mistached timestamp length: {len(trigger_ts)} vs {dshape}")
-                        spl = uspline(ts, gaussian_filter(trans_data, gf_sigma))
-                        ts0 = trigger_ts.flatten()
-                        trans_data1 = spl(ts0)
-                    else:
-                        raise Exception(f"timestamp data for {trigger} cannot be found.")
-
-                    # smoothing/interpolating
-                    if plot_trigger:
-                        plt.figure()
-                        plt.plot(ts, trans_data)
-                        plt.plot(ts0, trans_data1, "o")
-                    trans_data = trans_data1
+                try:
+                    trans_data = self.d0s[s]["transmitted"]
+                except:
+                    print("attempting to run get_mon() first ...")
+                    self.get_mon(sn=s, trigger=trigger)
+                    trans_data = self.d0s[s]["transmitted"]
 
             # these are the datasets that needs updated trans values
             # also need to rescale merged data, in case it's been scale, e.g. during normalization
@@ -1021,17 +1186,21 @@ class h5xs():
             return
         
         if sn not in list(self.attrs.keys()):
-            self.d1s[sn] = {}
+            self.d0s[sn] = {}    # these are attributes extracted from each data point
+            self.d1s[sn] = {}    # 1d scattering data, of the type data1d
+            self.d2s[sn] = {}    # 2d data, of the type MatrixWithCorrds
             self.attrs[sn] = {}
         grp = fh5[sn+'/processed']
         for k in list(grp.attrs.keys()):
             self.attrs[sn][k] = grp.attrs[k]   
+        # initially only handles 1d data
         for k in lsh5(grp, top_only=True, silent=True):
             if 'trans' in grp[k].attrs.keys():
                 tvs = grp[k].attrs['trans']
             else:
                 tvs = 0
             self.d1s[sn][k] = unpack_d1(grp[k], self.qgrid, sn+k, tvs)   
+            
 
     def save_d1s(self, sn=None, debug=False):
         """
