@@ -197,7 +197,7 @@ class Data1d:
                 print("normalized to %f" % ref_trans)
 
 
-    def avg(self, dsets, weighted=False, qmax_for_weight=0.3, 
+    def avg(self, dsets, weighted=True, qmax_for_weight=0.3, 
             plot_data=False, ax=None, debug=False, fontsize='large'):
         """
         dsets is a collection of Data1d
@@ -414,7 +414,7 @@ class Data1d:
                 
         return self
 
-    def merge(self, d1, qmax=-1, qmin=-1, fix_scale=-1, debug=False):
+    def merge(self, d1, qmax=-1, qmin=-1, fix_scale=-1, debug=False, create_new=False):
         """
         combine the data in self and d1
         scale d1 intensity to match self
