@@ -3,6 +3,48 @@ import matplotlib.colors as mc
 from functools import reduce
 import subprocess
 
+
+def calc_avg(dat:list, err:list, method="simple"):
+    """ both dat and err should be lists of numpy arrays, corresponding to data and error bar
+        calculate weighted average the data and the corresponding error 
+        use 1/err^2 as the weight
+    """
+    if not method in ["simple", "err_weighted"]:
+        raise Exception("method must be either 'simple' or 'err_weighted'.")
+    
+    da = np.zeros_like(dat[0])
+    ea = np.zeros_like(dat[0])
+    wt = np.zeros_like(dat[0])
+    
+    if method=="err_weighted":
+        if len(err)==0:
+            raise Exception("error bars required for err_weighted average.")
+        for i in range(len(dat)):
+            idx = (err[i]>0)
+            wt[~idx] = 0 
+            wt[idx] = 1./err[i][idx]**2
+            da[idx] += wt[idx]*dat[i][idx]
+            ea[idx] += wt[idx]
+        idx = (ea>0)
+        da[idx] /= ea[idx]
+        ea[idx] = 1./np.sqrt(ea[idx])
+    else:
+        for i in range(len(dat)):
+            idx = ~np.isnan(dat[i])
+            da[idx] += dat[i][idx]
+            if len(err)>0:
+                ea[idx] += err[i][idx]
+            wt[idx] += 1.
+        idx = (wt>0)
+        da[idx] /= wt[idx]
+        if len(err)>0:
+            ea[idx] = ea[idx]/wt[idx]/np.sqrt(wt[idx])        
+        else: 
+            ea = None
+
+    return da,ea
+
+
 def max_len(d1a, d1b, return_all=False):
     """ perform a Cormap-like pairwise comparison between 2 arrays
         if return_all is True, return a sequence of True/False base on the compasion of 
@@ -109,7 +151,7 @@ def cmap_map(function, cmap):
     return mc.LinearSegmentedColormap('colormap', cdict, 1024)
 
 
-def smooth(x,half_window_len=11,window='hanning'):
+def smooth(x, half_window_len=11, window='hanning'):
     """smooth the data using a window with requested size.
     revised from numpy cookbook, https://scipy-cookbook.readthedocs.io/items/SignalSmooth.html
     
